/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
* File Name    : comp.c
* Version      : Applilet EZ PL for RL78
* Device(s)    : R5F1056A,R5F1057A,R5F1058A
* Tool-Chain   : CC-RL
* Description  : This file implements main function.
* Creation Date: 2016/09/14
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "sfr.h"
#include "common.h"
#include "config.h"
#include "comp.h"

#if defined COMP_USED
/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
static UCHAR	ucgCOMP_Val[2] = {0, 0};
static UCHAR	ucgCOMP_Cnt = 0;

/*******************************************************************************
* Function Name: COMP_Set
* Description  : COMPARATOR panel parameter setting.
* Arguments    : In7    	... ON/OFF
*                In6 	... ON/OFF
*                In5 	... ON/OFF
*                In4 	... ON/OFF
*                In3 	... ON/OFF
*                In2 	... ON/OFF
*                In1 	... ON/OFF
*                In0 	... ON/OFF
* Return Value : none
********************************************************************************/
void COMP_Set( UCHAR In7, UCHAR In6, UCHAR In5, UCHAR In4, UCHAR In3, UCHAR In2, UCHAR In1, UCHAR In0 )
{
	UCHAR ucVal;
	ucVal = ((In7 & 0x01)<<7) | ((In6 & 0x01)<<6) | ((In5 & 0x01)<<5) | ((In4 & 0x01)<<4) |
			((In3 & 0x01)<<3) | ((In2 & 0x01)<<2) | ((In1 & 0x01)<<1) | (In0 & 0x01);   
	
	if(ucgCOMP_Cnt < 2){
		ucgCOMP_Val[ucgCOMP_Cnt] = ucVal;
		ucgCOMP_Cnt ++;
	}
}

/*******************************************************************************
* Function Name: COMP_Proc
* Description  : COMPARATOR panel compare processing.
* Arguments    : Big    	... OutBig-terminal status
*                Equal 	... OutEqual-terminal status
*                Small 	... OutSmall-terminal status
* Return Value : none
********************************************************************************/
void COMP_Proc( UCHAR* Big, UCHAR* Equal, UCHAR* Small )
{
	if(ucgCOMP_Val[0] > ucgCOMP_Val[1]){
		*Big = HIGH;
		*Equal = LOW;
		*Small = LOW;

	}else if(ucgCOMP_Val[0] < ucgCOMP_Val[1]){
		*Big = LOW;
		*Equal = LOW;
		*Small = HIGH;
		
	}else if(ucgCOMP_Val[0] == ucgCOMP_Val[1]){
		*Big = LOW;
		*Equal = HIGH;
		*Small = LOW;
	}

	ucgCOMP_Cnt = 0;
	ucgCOMP_Val[0] = 0;
	ucgCOMP_Val[1] = 0;
}

/*******************************************************************************
* Function Name: COMP_Initialize
* Description  : COMPARATOR panel initialized.
* Arguments    : none
* Return Value : none
********************************************************************************/
void COMP_Initialize( void )
{
	ucgCOMP_Cnt = 0;
}
#endif /* end of defined COMP_USED */

