/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
* File Name    : standbycommon.c
* Version      : Applilet EZ PL for RL78
* Device(s)    : R5F1056A,R5F1057A,R5F1058A
* Tool-Chain   : CC-RL
* Description  : This file implements the function of Standby common.
* Creation Date: 2016/09/14
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "sfr.h"
#include "common.h"
#include "config.h"
#include "standbycommon.h"

#if defined STANDBY_USED
/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
USHORT usDODSR_init;
USHORT usDODSR_max_init;
UCHAR  ucDOCR_add_init;
UCHAR  ucDOCR_sub_init;
USHORT usDOC_sub_result;
UCHAR  ucADM2_snooze_off;
UCHAR  ucADM2_snooze_on;
USHORT usADC_Result;

/*******************************************************************************
Private variables and functions
*******************************************************************************/
void switch_flashmode_to_LS( void );
void switch_flashmode_to_LP( void );
void change_operation_clock( void );

UCHAR	ucgTE0;
USHORT	usgSCR00;
USHORT	usgSCR01;
USHORT	usgSCR02;
USHORT	usgSCR03;
USHORT	usgMK0;
USHORT	usgMK1;
USHORT	usgMK2;
UCHAR	ucgEGP0;
UCHAR	ucgEGN0;
UCHAR	ucgEGP1;
UCHAR	ucgEGN1;

/*******************************************************************************
* Function Name: FLASHMODE_set_rower
* Description  : switch the flash operation mode.
* Arguments    : none
* Return Value : none
********************************************************************************/
void FLASHMODE_set_rower( void )
{
	switch (SYS_FLASHMODE)
	{
		case FLASHMODE_HS:
			switch_flashmode_to_LS();
			break;
		case FLASHMODE_LS:
			switch_flashmode_to_LP();
			break;
		case FLASHMODE_LV:
			change_operation_clock();
			break;
		default:
			break;
	}
}

/*******************************************************************************
* Function Name: switch_flashmode_to_LS
* Description  : switch the flash operation mode.
* Arguments    : none
* Return Value : none
********************************************************************************/
void switch_flashmode_to_LS( void )
{
	switch (SYS_CLOCK_SRC)
	{
		case CLOCK_SRC_HIGHSPEED:
			if (HOCODIV < 0x02) {
				/* clock is set to 8MHz or 6MHz. */
				HOCODIV = 0x02;
			}
			
			break;
			
		case CLOCK_SRC_MIDDLESPEED:
		
			/* Clock changing: change to HOCO, and stop MOCO. */
			HOCODIV = 0x05;				/* clock is set to 1MHz. */
			HIOSTOP = 0;				/* HOCO operation start */
			MCM1 = 0;					/* Set fOCO to HOCO. */
			while (MCS1 != 0)			/* Wait fOCO to be set to HOCO. */
			{
				NOP();
			}
			MIOEN  = 0;					/* MOCO operation stop */
			
			break;
			
		case CLOCK_SRC_X1:
		case CLOCK_SRC_EXTERNAL:
			/* Clock changing: change to HOCO. */
			if (SYS_CLOCK_FREQ < 8) {
				/* clock is set to 8MHz. */
				HOCODIV = 0x02;
			} else {
				/* clock is set to 1MHz. */
				HOCODIV = 0x05;
			}
			HIOSTOP = 0;
			CKC = 0x00;
			while (MCS != 0)		/* Wait Main Clock to be set. */
			{
				NOP();
			}
			
			break;
			
		default:
			break;
	}
	
	/* Regulator Mode is "Normal setting". */
	MCSEL = 0;
	
	/* switch the flash operation mode. */
	FLMWEN = 1;
	FLMODE = 0xC0;					/* HS mode */
	FLMWEN = 0;
	
}

/*******************************************************************************
* Function Name: switch_flashmode_to_LS
* Description  : switch the flash operation mode.
* Arguments    : none
* Return Value : none
********************************************************************************/
void switch_flashmode_to_LP( void )
{
	
	switch (SYS_CLOCK_SRC)
	{
		case CLOCK_SRC_HIGHSPEED:
			
			if (SYS_CLOCK_FREQ == 6 || SYS_CLOCK_FREQ == 3) {
				/* clock is set to 3MHz. */
				HOCODIV = 0x03;
				/* LP mode can't be used. */
				return;
				
			} else {
				/* clock is set to 1MHz. */
				HOCODIV = 0x05;
			}
			
			break;
			
		case CLOCK_SRC_MIDDLESPEED:
		
			/* Clock changing: change to HOCO, and stop MOCO. */
			HOCODIV = 0x05;				/* clock is set to 1MHz. */
			HIOSTOP = 0;				/* HOCO operation start */
			MCM1 = 0;					/* Set fOCO to HOCO. */
			while (MCS1 != 0)			/* Wait fOCO to be set to HOCO. */
			{
				NOP();
			}
			MIOEN  = 0;					/* MOCO operation stop */
			
			break;
			
		case CLOCK_SRC_X1:
		case CLOCK_SRC_EXTERNAL:
			/* Clock changing: change to HOCO. */
			/* clock is set to 1MHz. */
			HOCODIV = 0x05;
			HIOSTOP = 0;
			MCM0 = 0;
			while (MCS != 0)		/* Wait Main Clock to be set. */
			{
				NOP();
			}
			
			break;
			
		default:
			break;
	}
	
	/* Regulator Mode is "Normal setting" for the meantime. */
	MCSEL = 0;

	/* switch the flash operation mode. */
	FLMWEN = 1;
	FLMODE = 0x40;				/* LP mode */
	FLMWEN = 0;

	/* Regulator Mode is "Low-power consumption setting" as the original target. */
	MCSEL = 1;
}

/*******************************************************************************
* Function Name: change_operation_clock
* Description  : Operation clock change.
* Arguments    : none
* Return Value : none
********************************************************************************/
void change_operation_clock( void )
{
	
	switch (SYS_CLOCK_SRC)
	{
		case CLOCK_SRC_HIGHSPEED:
			
			if ( SYS_CLOCK_FREQ != 3 )
			{
				/* clock is set to 1MHz. */
				HOCODIV = 0x05;
			}
			
			break;
			
		case CLOCK_SRC_MIDDLESPEED:
		
			/* Clock changing: change to HOCO, and stop MOCO. */
			HOCODIV = 0x05;				/* clock is set to 1MHz. */
			HIOSTOP = 0;				/* HOCO operation start */
			MCM1 = 0;					/* Set fOCO to HOCO. */
			while (MCS1 != 0)			/* Wait fOCO to be set to HOCO. */
			{
				NOP();
			}
			MIOEN  = 0;					/* MOCO operation stop */
			
			break;
			
		case CLOCK_SRC_X1:
		case CLOCK_SRC_EXTERNAL:
			
			/* Clock changing: change to HOCO. */
			HOCODIV = 0x05;			/* clock is set to 1MHz. */
			HIOSTOP = 0;
			CKC = 0x00;
			while (MCS != 0)		/* Wait Main Clock to be set. */
			{
				NOP();
			}
			
			break;
			
		default:
			break;
	}
}

/*******************************************************************************
* Function Name: FLASHMODE_return_beginning
* Description  : Flash Mode return it to the beginning.
* Arguments    : none
* Return Value : none
********************************************************************************/
void FLASHMODE_return_beginning( void )
{
	/* Regulator Mode is "Normal setting" for the meantime. */
	MCSEL = 0;
	
	/* switch the flash operation mode. */
	switch (SYS_FLASHMODE)
	{
		case FLASHMODE_HS:
			
			FLMWEN = 1;
			FLMODE = 0xC0;		/* HS mode */
			FLMWEN = 0;
			
			break;
			
		case FLASHMODE_LS:
			
			FLMWEN = 1;
			FLMODE = 0x80;		/* LS mode */
			FLMWEN = 0;
			
			break;
			
		case FLASHMODE_LV:
			
			FLMWEN = 1;
			FLMODE = 0x00;		/* LV mode */
			FLMWEN = 0;
			
			break;
			
		default:
			break;
		
	}
	
	/* Clock changing */
	switch (SYS_CLOCK_SRC)
	{
		case CLOCK_SRC_HIGHSPEED:
			
			switch (SYS_CLOCK_FREQ)
			{
				case 24:
					/* clock is set to 24MHz. */
					HOCODIV = 0x00;
					
					break;
					
				case 16:
				case 12:
					/* clock is set to 16MHz or 12MHz. */
					HOCODIV = 0x01;
					
					break;
					
				case 8:
				case 6:
					/* clock is set to 8MHz or 6MHz. */
					HOCODIV = 0x02;
					
					break;
					
				case 4:
				case 3:
					/* clock is set to 4MHz or 3MHz. */
					HOCODIV = 0x03;
					
					break;
					
				case 2:
					/* clock is set to 2MHz */
					HOCODIV = 0x04;
					
					break;
					
				case 1:
					/* clock is set to 1MHz. */
					HOCODIV = 0x05;
					
					break;
					
				default:
					break;
			}
			
			break;
			
		case CLOCK_SRC_MIDDLESPEED:
			
			switch (SYS_CLOCK_FREQ)
			{
				case 4:
					/* clock is set to 4MHz. */
					MOCODIV = 0x00;
					
					break;
					
				case 2:
					/* clock is set to 2MHz. */
					MOCODIV = 0x01;
					
					break;
					
				case 1:
					/* clock is set to 1MHz. */
					MOCODIV = 0x02;
					
					break;
					
				default:
					break;
			}
			
			/* Clock changing: change to MOCO. */
			MIOEN = 1;					/* MOCO operation start */
			MCM1 = 1;					/* Set fOCO to MOCO. */
			while (MCS1 != 1)			/* Wait fOCO to be set to MOCO. */
			{
				NOP();
			}
			HIOSTOP  = 1;				/* HOCO operation stop */
			
			break;
			
		case CLOCK_SRC_X1:
		case CLOCK_SRC_EXTERNAL:
			
			if ( SYS_CLOCK_SRC == CLOCK_SRC_X1 ) {
				while(OSTC != 0xFF)
				{
					NOP();
				}
			}
			MCM0 = 1;
			while (MCS != 1)		/* Wait Main Clock to be set. */
			{
				NOP();
			}
			HIOSTOP  = 1;			/* HOCO operation stop */
			
			break;
			
		default:
			break;
	}
	
}

/*******************************************************************************
* Function Name: standby_before_state_save
* Description  : State save before a standby.
* Arguments    : none
* Return Value : none
********************************************************************************/
void standby_before_state_save( void )
{
#ifdef ELC_USED
	volatile UCHAR * addr;
	USHORT  count;
#endif
	
	/* Timer STOP */
	ucgTE0 = TE0;									/* Timer STATUS */
	TT0 = ucgTE0;									/* Timer STOP */
	
	/* Serial STOP */
	usgSCR00 = SCR00 & 0xC000;						/* Serial CH0 STATUS(TXE00, RXE00) */
	usgSCR01 = SCR01 & 0xC000;						/* Serial CH1 STATUS(TXE01, RXE01) */
	usgSCR02 = SCR02 & 0xC000;						/* Serial CH1 STATUS(TXE02, RXE02) */
	usgSCR03 = SCR03 & 0xC000;						/* Serial CH1 STATUS(TXE03, RXE03) */
	if ( usgSCR00 != 0x0000 )
	{
		SCR00 &= 0x3FFF;							/* Serial CH0 STOP(TXE00=0,RXE00=0) */
	}
	if ( usgSCR01 != 0x0000 )
	{
		SCR01 &= 0x3FFF;							/* Serial CH1 STOP(TXE01=0,RXE01=0) */
	}
	if ( usgSCR02 != 0x0000 )
	{
		SCR02 &= 0x3FFF;							/* Serial CH2 STOP(TXE02=0,RXE02=0) */
	}
	if ( usgSCR03 != 0x0000 )
	{
		SCR03 &= 0x3FFF;							/* Serial CH3 STOP(TXE03=0,RXE03=0) */
	}
	
#ifdef ANI3_USED
	ADCRES = 1;		/* reset A/D converter */
	ADCRES = 0;		/* reset release of A/D converter */
	
	CMPRES = 1;		/* reset Comparator */
	CMPRES = 0;		/* reset release of Comparator */
	
	DACRES = 1;		/* reset D/A converter */
	DACRES = 0;		/* reset release of D/A converter */
#endif
	

#ifdef DTC_USED
	/* disable DTC module */
	DTCEN0 = 0x00;
	DTCEN1 = 0x00;
	DTCEN2 = 0x00;
#endif

#ifdef ELC_USED
	/* Clear ELC module */
	addr = &ELSELR00;
	
	for (count = 0U; count < ELC_DESTINATION_COUNT; count++)
	{
		*addr = ELC_EVENT_LINK_OFF;
		addr++;
	}
#endif
	
	/* External interrupt disable */
	ucgEGP0 = EGP0;
	ucgEGN0 = EGN0;
	EGP0 = 0x00;
	EGN0 = 0x00;
	ucgEGP1 = EGP1;
	ucgEGN1 = EGN1;
	EGP1 = 0x00;
	EGN1 = 0x00;
	
	/* All individual interrupt is save and prohibited	*/
	usgMK0 = MK0;
	usgMK1 = MK1;
	usgMK2 = MK2;
	MK0 = 0xFFFF;
	IF0 = 0x0000;
	MK1 = 0xFFFF;
	IF1 = 0x0000;
	MK2 = 0xFFFF;
	IF2 = 0x0000;
}

/*******************************************************************************
* Function Name: standby_before_state_save
* Description  : State return after a standby.
* Arguments    : none
* Return Value : none
********************************************************************************/
void standby_after_state_return( void )
{
	/* Timer START */
	TS0 |= ucgTE0;									/* Timer START */
	
	/* Serial START */
	if( usgSCR00 != 0x0000 )
	{
		SCR00 |= usgSCR00;							/* Serial CH0 return */
	}
	if( usgSCR01 != 0x0000 )
	{
		SCR01 |= usgSCR01;							/* Serial CH1 return */
	}
	if( usgSCR02 != 0x0000 )
	{
		SCR02 |= usgSCR02;							/* Serial CH2 return */
	}
	if( usgSCR03 != 0x0000 )
	{
		SCR03 |= usgSCR03;							/* Serial CH3 return */
	}
	
	/* External interrupt enable */
	EGP0 = ucgEGP0;
	EGN0 = ucgEGN0;
	EGP1 = ucgEGP1;
	EGN1 = ucgEGN1;
	
	/* individual interrupt is return */
	MK0 = usgMK0;
	IF0 = 0x0000;
	MK1 = usgMK1;
	IF1 = 0x0000;
	MK2 = usgMK2;
	IF2 = 0x0000;
}
#endif /* end of defined STANDBY_USED */
